@@ -35,6 +35,8 @@ import (
 	"github.com/pkg/errors"
 	"github.com/pulumi/pulumi/pkg/v3/codegen"
 	"github.com/pulumi/pulumi/pkg/v3/codegen/schema"
+	"github.com/pulumi/pulumi/sdk/v3/go/common/diag"
+	"github.com/pulumi/pulumi/sdk/v3/go/common/util/cmdutil"
 	"github.com/pulumi/pulumi/sdk/v3/go/common/util/contract"
 )
 
@@ -1017,6 +1019,21 @@ func (mod *modContext) genResource(w io.Writer, r *schema.Resource) error {
 		fmt.Fprintf(w, "                },\n")
 	}
 
+	replaceOnChangesProps, errList := r.ReplaceOnChanges()
+	for _, err := range errList {
+		cmdutil.Diag().Warningf(&diag.Diag{Message: err.Error()})
+	}
+	if len(replaceOnChangesProps) > 0 {
+		fmt.Fprint(w, "                ReplaceOnChanges =\n")
+		fmt.Fprintf(w, "                {\n")
+		for _, n := range schema.PropertyListJoinToString(replaceOnChangesProps,
+			func(s string) string { return s }) {
+			fmt.Fprintf(w, "                    ")
+			fmt.Fprintf(w, "%q,\n", n)
+		}
+		fmt.Fprintf(w, "                },\n")
+	}
+
 	fmt.Fprintf(w, "            };\n")
 	fmt.Fprintf(w, "            var merged = %s.Merge(defaultOptions, options);\n", optionsType)
 	fmt.Fprintf(w, "            // Override the ID if one was specified for consistency with other language SDKs.\n")
@@ -1962,6 +1979,13 @@ func genPackageMetadata(pkg *schema.Package, assemblyName string, packageReferen
 
 	files.add(assemblyName+".csproj", projectFile)
 	files.add("logo.png", logo)
+
+	// Generate version.txt
+	versionFile := "version.txt"
+	if _, exists := files[versionFile]; !exists && pkg.Version != nil {
+		files[versionFile] = []byte(pkg.Version.String())
+	}
+
 	return nil
 }
 
@@ -2212,8 +2236,8 @@ func GeneratePackage(tool string, pkg *schema.Package, extraFiles map[string][]b
 	files := fs{}
 	for p, f := range extraFiles {
 		files.add(p, f)
-
 	}
+
 	for _, mod := range modules {
 		if err := mod.gen(files); err != nil {
 			return nil, err
