@@ -3,15 +3,33 @@ package dotnet
 import (
 	"testing"
 
-	"github.com/pulumi/pulumi/pkg/v3/codegen/internal/test"
-	"github.com/pulumi/pulumi/pkg/v3/codegen/schema"
-
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+
+	"github.com/pulumi/pulumi/pkg/v3/codegen/internal/test"
+	"github.com/pulumi/pulumi/pkg/v3/codegen/schema"
+	"github.com/pulumi/pulumi/pkg/v3/testing/integration"
+	"github.com/pulumi/pulumi/sdk/v3/go/common/util/executable"
 )
 
 func TestGeneratePackage(t *testing.T) {
-	test.TestSDKCodegen(t, "dotnet", GeneratePackage)
+	test.TestSDKCodegen(t, &test.SDKCodegenOptions{
+		Language:   "dotnet",
+		GenPackage: GeneratePackage,
+		Checks: map[string]test.CodegenCheck{
+			"dotnet/compile": typeCheckGeneratedPackage,
+		},
+	})
+}
+
+func typeCheckGeneratedPackage(t *testing.T, pwd string) {
+	var err error
+	var dotnet string
+	dotnet, err = executable.FindExecutable("dotnet")
+	require.NoError(t, err)
+	cmdOptions := integration.ProgramTestOptions{}
+	err = integration.RunCommand(t, "dotnet build", []string{dotnet, "build"}, pwd, &cmdOptions)
+	require.NoError(t, err)
 }
 
 func TestGenerateType(t *testing.T) {
