@@ -149,7 +149,7 @@ func (pkg *pkgContext) tokenToType(tok string) string {
 		newName, renamed := modPkg.renamed[name]
 		if renamed {
 			name = newName
-		} else if modPkg.duplicateTokens[tok] {
+		} else if modPkg.duplicateTokens[strings.ToLower(tok)] {
 			// maintain support for duplicate tokens for types and resources in Kubernetes
 			name = name + "Type"
 		}
@@ -2514,10 +2514,10 @@ func generatePackageContextMap(tool string, pkg *schema.Package, goInfo GoPackag
 	// compute set of names generated by a resource
 	// handling any potential collisions via remapping along the way
 	scanResource := func(r *schema.Resource) {
-		if resSeen[r.Token] {
+		if resSeen[strings.ToLower(r.Token)] {
 			return
 		}
-		resSeen[r.Token] = true
+		resSeen[strings.ToLower(r.Token)] = true
 		pkg := getPkgFromToken(r.Token)
 		pkg.resources = append(pkg.resources, r)
 		pkg.schemaNames.Add(tokenToName(r.Token))
@@ -2598,14 +2598,14 @@ func generatePackageContextMap(tool string, pkg *schema.Package, goInfo GoPackag
 		case *schema.ObjectType:
 			pkg := getPkgFromToken(t.Token)
 			// maintain support for duplicate tokens for types and resources in Kubernetes
-			if resSeen[t.Token] {
-				pkg.duplicateTokens[t.Token] = true
+			if resSeen[strings.ToLower(t.Token)] {
+				pkg.duplicateTokens[strings.ToLower(t.Token)] = true
 				return
 			}
-			if typeSeen[t.Token] {
+			if typeSeen[strings.ToLower(t.Token)] {
 				return
 			}
-			typeSeen[t.Token] = true
+			typeSeen[strings.ToLower(t.Token)] = true
 
 			name := pkg.tokenToType(t.Token)
 			getNames := func(suffix string) []string {
