@@ -928,10 +928,7 @@ func (mod *modContext) genTypes(dir string, fs fs) error {
 	if err := genTypes("_inputs.py", true); err != nil {
 		return err
 	}
-	if err := genTypes("outputs.py", false); err != nil {
-		return err
-	}
-	return nil
+	return genTypes("outputs.py", false)
 }
 
 func awaitableTypeNames(tok string) (baseName, awaitableName string) {
@@ -1896,9 +1893,20 @@ func genPackageMetadata(
 	fmt.Fprintf(w, "from subprocess import check_call\n")
 	fmt.Fprintf(w, "\n\n")
 
-	// Create a constant for the version number to replace during build
-	fmt.Fprintf(w, "VERSION = \"0.0.0\"\n")
-	fmt.Fprintf(w, "PLUGIN_VERSION = \"0.0.0\"\n\n")
+	// If `pkg` supplies a version, use that. Otherwise, create a constant for
+	// the version number to replace during build.
+	packageVersion := "0.0.0"
+	if pkg.Version != nil {
+		packageVersion = pythonVersion(pkg.Version)
+	}
+	pluginVersion := "0.0.0"
+	if pkg.Version != nil {
+		// This happens in test builds
+		pluginVersion = pkg.Version.String()
+	}
+
+	fmt.Fprintf(w, "VERSION = %q\n", packageVersion)
+	fmt.Fprintf(w, "PLUGIN_VERSION = %q\n\n", pluginVersion)
 
 	// Create a command that will install the Pulumi plugin for this resource provider.
 	fmt.Fprintf(w, "class InstallPluginCommand(install):\n")
