@@ -184,9 +184,11 @@ func functionName(tokenArg model.Expression) (string, string, string, hcl.Diagno
 var functionImports = map[string]string{
 	"fileArchive": "pulumi",
 	"fileAsset":   "pulumi",
+	"filebase64":  "base64",
 	"readDir":     "os",
 	"toBase64":    "base64",
 	"toJSON":      "json",
+	"sha1":        "hashlib",
 }
 
 func (g *generator) getFunctionImports(x *model.FunctionCallExpression) string {
@@ -218,6 +220,8 @@ func (g *generator) GenFunctionCallExpression(w io.Writer, expr *model.FunctionC
 		g.Fgenf(w, "pulumi.FileArchive(%.v)", expr.Args[0])
 	case "fileAsset":
 		g.Fgenf(w, "pulumi.FileAsset(%.v)", expr.Args[0])
+	case "filebase64":
+		g.Fgenf(w, "(lambda path: base64.b64encode(open(path).read().encode()).decode())(%.v)", expr.Args[0])
 	case hcl2.Invoke:
 		pkg, module, fn, diags := functionName(expr.Args[0])
 		contract.Assert(len(diags) == 0)
@@ -298,6 +302,8 @@ func (g *generator) GenFunctionCallExpression(w io.Writer, expr *model.FunctionC
 		g.Fgenf(w, "base64.b64encode(%.16v.encode()).decode()", expr.Args[0])
 	case "toJSON":
 		g.Fgenf(w, "json.dumps(%.v)", expr.Args[0])
+	case "sha1":
+		g.Fgenf(w, "hashlib.sha1(%v.encode()).hexdigest()", expr.Args[0])
 	default:
 		var rng hcl.Range
 		if expr.Syntax != nil {
