@@ -1,4 +1,4 @@
-// Copyright 2016-2020, Pulumi Corporation.
+// Copyright 2016-2021, Pulumi Corporation.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -16,8 +16,10 @@ package schema
 
 import (
 	"bytes"
+	_ "embed"
 	"encoding/json"
 	"fmt"
+	"io"
 	"math"
 	"net/url"
 	"os"
@@ -29,6 +31,7 @@ import (
 	"github.com/blang/semver"
 	"github.com/hashicorp/hcl/v2"
 	"github.com/pkg/errors"
+	"github.com/santhosh-tekuri/jsonschema/v5"
 	"gopkg.in/yaml.v3"
 
 	"github.com/pulumi/pulumi/sdk/v3/go/common/resource/plugin"
@@ -403,7 +406,7 @@ type Resource struct {
 //
 // Resource A {
 // Properties: {
-// 	 Resource B {
+// 	 Object B {
 // 	   Object D: {
 // 	     ReplaceOnChanges: true
 // 	     }
@@ -461,6 +464,10 @@ func replaceOnChangesType(t Type, stack *map[string]struct{}) ([][]*Property, []
 				err = append(err, errors.Errorf("Found recursive object %q", p.Name))
 			}
 		}
+		// We don't want to emit errors where replaceOnChanges is not used.
+		if len(changes) == 0 {
+			return nil, nil
+		}
 		return changes, err
 	} else if a, ok := t.(*ArrayType); ok {
 		// This looks for types internal to the array, not a property of the array.
@@ -469,7 +476,7 @@ func replaceOnChangesType(t Type, stack *map[string]struct{}) ([][]*Property, []
 		// This looks for types internal to the map, not a property of the array.
 		return replaceOnChangesType(m.ElementType, stack)
 	}
-	return [][]*Property{}, []error{}
+	return nil, nil
 }
 
 // Joins the output of `ReplaceOnChanges` into property path names.
@@ -1266,6 +1273,22 @@ func (m *RawMessage) UnmarshalYAML(node *yaml.Node) error {
 	return nil
 }
 
+//go:embed pulumi.json
+var metaSchema []byte
+
+var MetaSchema *jsonschema.Schema
+
+func init() {
+	compiler := jsonschema.NewCompiler()
+	compiler.LoadURL = func(u string) (io.ReadCloser, error) {
+		if u == "blob://pulumi.json" {
+			return io.NopCloser(bytes.NewReader(metaSchema)), nil
+		}
+		return jsonschema.LoadURL(u)
+	}
+	MetaSchema = compiler.MustCompile("blob://pulumi.json")
+}
+
 // TypeSpec is the serializable form of a reference to a type.
 type TypeSpec struct {
 	// Type is the primitive or composite type, if any. May be "bool", "integer", "number", "string", "array", or
@@ -1496,6 +1519,39 @@ func errorf(path, message string, args ...interface{}) *hcl.Diagnostic {
 	}
 }
 
+func validateSpec(spec PackageSpec) (hcl.Diagnostics, error) {
+	bytes, err := json.Marshal(spec)
+	if err != nil {
+		return nil, err
+	}
+	var raw interface{}
+	if err = json.Unmarshal(bytes, &raw); err != nil {
+		return nil, err
+	}
+
+	if err = MetaSchema.Validate(raw); err == nil {
+		return nil, nil
+	}
+	validationError, ok := err.(*jsonschema.ValidationError)
+	if !ok {
+		return nil, err
+	}
+
+	var diags hcl.Diagnostics
+	var appendError func(err *jsonschema.ValidationError)
+	appendError = func(err *jsonschema.ValidationError) {
+		if err.InstanceLocation != "" && err.Message != "" {
+			diags = diags.Append(errorf("#"+err.InstanceLocation, "%v", err.Message))
+		}
+		for _, err := range err.Causes {
+			appendError(err)
+		}
+	}
+	appendError(validationError)
+
+	return diags, nil
+}
+
 // bindSpec converts a serializable PackageSpec into a Package. This function includes a loader parameter which
 // works as a singleton -- if it is nil, a new loader is instantiated, else the provided loader is used. This avoids
 // breaking downstream consumers of ImportSpec while allowing us to extend schema support to external packages.
@@ -1516,6 +1572,18 @@ func errorf(path, message string, args ...interface{}) *hcl.Diagnostic {
 func bindSpec(spec PackageSpec, languages map[string]Language, loader Loader) (*Package, hcl.Diagnostics, error) {
 	var diags hcl.Diagnostics
 
+	// Validate the package against the metaschema.
+	validationDiags, err := validateSpec(spec)
+	if err != nil {
+		return nil, nil, fmt.Errorf("validating spec: %w", err)
+	}
+	diags = diags.Extend(validationDiags)
+
+	// Validate that there is a name
+	if spec.Name == "" {
+		diags = diags.Append(errorf("#/name", "no name provided"))
+	}
+
 	// Parse the version, if any.
 	var version *semver.Version
 	if spec.Version != "" {
@@ -1986,6 +2054,9 @@ func (t *types) bindType(path string, spec TypeSpec, inputShape bool) (result Ty
 		var discriminator string
 		var mapping map[string]string
 		if spec.Discriminator != nil {
+			if spec.Discriminator.PropertyName == "" {
+				diags = diags.Append(errorf(path, "discriminator must provide a property name"))
+			}
 			discriminator = spec.Discriminator.PropertyName
 			mapping = spec.Discriminator.Mapping
 		}
@@ -2131,6 +2202,9 @@ func bindDefaultValue(path string, value interface{}, spec *DefaultSpec, typ Typ
 		for name, raw := range spec.Language {
 			language[name] = json.RawMessage(raw)
 		}
+		if len(spec.Environment) == 0 {
+			diags = diags.Append(errorf(path, "Default must specify an environment"))
+		}
 
 		dv.Environment, dv.Language = spec.Environment, language
 	}
