@@ -1,4 +1,4 @@
-// Copyright 2016-2018, Pulumi Corporation.
+// Copyright 2016-2021, Pulumi Corporation.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -15,6 +15,7 @@
 package plugin
 
 import (
+	"fmt"
 	"reflect"
 	"sort"
 
@@ -38,6 +39,7 @@ type MarshalOptions struct {
 	RejectAssets       bool   // true if we should return errors on Asset and Archive values.
 	KeepResources      bool   // true if we are keeping resoures (otherwise we return raw urn).
 	SkipInternalKeys   bool   // true to skip internal property keys (keys that start with "__") in the resulting map.
+	KeepOutputValues   bool   // true if we are keeping output values.
 }
 
 const (
@@ -70,14 +72,14 @@ func MarshalProperties(props resource.PropertyMap, opts MarshalOptions) (*struct
 	for _, key := range props.StableKeys() {
 		v := props[key]
 		logging.V(9).Infof("Marshaling property for RPC[%s]: %s=%v", opts.Label, key, v)
-		if v.IsOutput() {
+		if v.IsOutput() && !v.OutputValue().Known && !opts.KeepOutputValues {
 			logging.V(9).Infof("Skipping output property for RPC[%s]: %v", opts.Label, key)
 		} else if opts.SkipNulls && v.IsNull() {
 			logging.V(9).Infof("Skipping null property for RPC[%s]: %s (as requested)", opts.Label, key)
 		} else if opts.SkipInternalKeys && resource.IsInternalPropertyKey(key) {
 			logging.V(9).Infof("Skipping internal property for RPC[%s]: %s (as requested)", opts.Label, key)
 		} else {
-			m, err := MarshalPropertyValue(v, opts)
+			m, err := MarshalPropertyValue(key, v, opts)
 			if err != nil {
 				return nil, err
 			} else if m != nil {
@@ -91,7 +93,8 @@ func MarshalProperties(props resource.PropertyMap, opts MarshalOptions) (*struct
 }
 
 // MarshalPropertyValue marshals a single resource property value into its "JSON-like" value representation.
-func MarshalPropertyValue(v resource.PropertyValue, opts MarshalOptions) (*structpb.Value, error) {
+func MarshalPropertyValue(key resource.PropertyKey, v resource.PropertyValue,
+	opts MarshalOptions) (*structpb.Value, error) {
 	if v.IsNull() {
 		return MarshalNull(opts), nil
 	} else if v.IsBool() {
@@ -111,7 +114,7 @@ func MarshalPropertyValue(v resource.PropertyValue, opts MarshalOptions) (*struc
 	} else if v.IsArray() {
 		var elems []*structpb.Value
 		for _, elem := range v.ArrayValue() {
-			e, err := MarshalPropertyValue(elem, opts)
+			e, err := MarshalPropertyValue(key, elem, opts)
 			if err != nil {
 				return nil, err
 			}
@@ -126,12 +129,12 @@ func MarshalPropertyValue(v resource.PropertyValue, opts MarshalOptions) (*struc
 		}, nil
 	} else if v.IsAsset() {
 		if opts.RejectAssets {
-			return nil, errors.New("unexpected Asset property value")
+			return nil, fmt.Errorf("unexpected Asset property value for %q", key)
 		}
 		return MarshalAsset(v.AssetValue(), opts)
 	} else if v.IsArchive() {
 		if opts.RejectAssets {
-			return nil, errors.New("unexpected Asset Archive property value")
+			return nil, fmt.Errorf("unexpected Asset Archive property value for %q", key)
 		}
 		return MarshalArchive(v.ArchiveValue(), opts)
 	} else if v.IsObject() {
@@ -142,34 +145,57 @@ func MarshalPropertyValue(v resource.PropertyValue, opts MarshalOptions) (*struc
 		return MarshalStruct(obj, opts), nil
 	} else if v.IsComputed() {
 		if opts.RejectUnknowns {
-			return nil, errors.New("unexpected unknown property value")
+			return nil, fmt.Errorf("unexpected unknown property value for %q", key)
 		} else if opts.KeepUnknowns {
 			return marshalUnknownProperty(v.Input().Element, opts), nil
 		}
 		return nil, nil // return nil and the caller will ignore it.
 	} else if v.IsOutput() {
-		// Note that at the moment we don't differentiate between computed and output properties on the wire.  As
-		// a result, they will show up as computed on the other end.  This distinction isn't currently interesting.
-		if opts.KeepUnknowns {
-			return marshalUnknownProperty(v.OutputValue().Element, opts), nil
+		if !opts.KeepOutputValues {
+			result := v.OutputValue().Element
+			if !v.OutputValue().Known {
+				// Unknown outputs are marshaled the same as Computed.
+				result = resource.MakeComputed(result)
+			}
+			if v.OutputValue().Secret {
+				result = resource.MakeSecret(result)
+			}
+			return MarshalPropertyValue(key, result, opts)
 		}
-		return nil, nil // return nil and the caller will ignore it.
+		obj := resource.PropertyMap{
+			resource.SigKey: resource.NewStringProperty(resource.OutputValueSig),
+		}
+		if v.OutputValue().Known {
+			obj["value"] = v.OutputValue().Element
+		}
+		if v.OutputValue().Secret {
+			obj["secret"] = resource.NewBoolProperty(v.OutputValue().Secret)
+		}
+		if len(v.OutputValue().Dependencies) > 0 {
+			deps := make([]resource.PropertyValue, len(v.OutputValue().Dependencies))
+			for i, dep := range v.OutputValue().Dependencies {
+				deps[i] = resource.NewStringProperty(string(dep))
+			}
+			obj["dependencies"] = resource.NewArrayProperty(deps)
+		}
+		output := resource.NewObjectProperty(obj)
+		return MarshalPropertyValue(key, output, opts)
 	} else if v.IsSecret() {
 		if !opts.KeepSecrets {
 			logging.V(5).Infof("marshalling secret value as raw value as opts.KeepSecrets is false")
-			return MarshalPropertyValue(v.SecretValue().Element, opts)
+			return MarshalPropertyValue(key, v.SecretValue().Element, opts)
 		}
 		secret := resource.NewObjectProperty(resource.PropertyMap{
 			resource.SigKey: resource.NewStringProperty(resource.SecretSig),
 			"value":         v.SecretValue().Element,
 		})
-		return MarshalPropertyValue(secret, opts)
+		return MarshalPropertyValue(key, secret, opts)
 	} else if v.IsResourceReference() {
 		ref := v.ResourceReferenceValue()
 		if !opts.KeepResources {
 			val := string(ref.URN)
 			if !ref.ID.IsNull() {
-				return MarshalPropertyValue(ref.ID, opts)
+				return MarshalPropertyValue(key, ref.ID, opts)
 			}
 			logging.V(5).Infof("marshalling resource value as raw URN or ID as opts.KeepResources is false")
 			return MarshalString(val, opts), nil
@@ -184,10 +210,11 @@ func MarshalPropertyValue(v resource.PropertyValue, opts MarshalOptions) (*struc
 		if ref.PackageVersion != "" {
 			m["packageVersion"] = resource.NewStringProperty(ref.PackageVersion)
 		}
-		return MarshalPropertyValue(resource.NewObjectProperty(m), opts)
+		return MarshalPropertyValue(key, resource.NewObjectProperty(m), opts)
 	}
 
-	contract.Failf("Unrecognized property value in RPC[%s]: %v (type=%v)", opts.Label, v.V, reflect.TypeOf(v.V))
+	contract.Failf("Unrecognized property value in RPC[%s] for %q: %v (type=%v)",
+		opts.Label, key, v.V, reflect.TypeOf(v.V))
 	return nil, nil
 }
 
@@ -242,7 +269,7 @@ func UnmarshalProperties(props *structpb.Struct, opts MarshalOptions) (resource.
 	// And now unmarshal every field it into the map.
 	for _, key := range keys {
 		pk := resource.PropertyKey(key)
-		v, err := UnmarshalPropertyValue(props.Fields[key], opts)
+		v, err := UnmarshalPropertyValue(pk, props.Fields[key], opts)
 		if err != nil {
 			return nil, err
 		} else if v != nil {
@@ -261,7 +288,8 @@ func UnmarshalProperties(props *structpb.Struct, opts MarshalOptions) (resource.
 }
 
 // UnmarshalPropertyValue unmarshals a single "JSON-like" value into a new property value.
-func UnmarshalPropertyValue(v *structpb.Value, opts MarshalOptions) (*resource.PropertyValue, error) {
+func UnmarshalPropertyValue(key resource.PropertyKey, v *structpb.Value,
+	opts MarshalOptions) (*resource.PropertyValue, error) {
 	contract.Assert(v != nil)
 
 	switch v.Kind.(type) {
@@ -279,7 +307,7 @@ func UnmarshalPropertyValue(v *structpb.Value, opts MarshalOptions) (*resource.P
 		s := v.GetStringValue()
 		if unk, isunk := unmarshalUnknownPropertyValue(s, opts); isunk {
 			if opts.RejectUnknowns {
-				return nil, errors.New("unexpected unknown property value")
+				return nil, fmt.Errorf("unexpected unknown property value for %q", key)
 			} else if opts.KeepUnknowns {
 				return &unk, nil
 			}
@@ -291,7 +319,7 @@ func UnmarshalPropertyValue(v *structpb.Value, opts MarshalOptions) (*resource.P
 		lst := v.GetListValue()
 		elems := make([]resource.PropertyValue, len(lst.GetValues()))
 		for i, elem := range lst.GetValues() {
-			e, err := UnmarshalPropertyValue(elem, opts)
+			e, err := UnmarshalPropertyValue(key, elem, opts)
 			if err != nil {
 				return nil, err
 			} else if e != nil {
@@ -323,7 +351,7 @@ func UnmarshalPropertyValue(v *structpb.Value, opts MarshalOptions) (*resource.P
 		switch sig {
 		case resource.AssetSig:
 			if opts.RejectAssets {
-				return nil, errors.New("unexpected Asset property value")
+				return nil, fmt.Errorf("unexpected Asset property value for %q", key)
 			}
 			asset, isasset, err := resource.DeserializeAsset(objmap)
 			if err != nil {
@@ -334,14 +362,14 @@ func UnmarshalPropertyValue(v *structpb.Value, opts MarshalOptions) (*resource.P
 			contract.Assert(isasset)
 			if opts.ComputeAssetHashes {
 				if err = asset.EnsureHash(); err != nil {
-					return nil, errors.Wrapf(err, "failed to compute asset hash")
+					return nil, errors.Wrapf(err, "failed to compute asset hash for %q", key)
 				}
 			}
 			m := resource.NewAssetProperty(asset)
 			return &m, nil
 		case resource.ArchiveSig:
 			if opts.RejectAssets {
-				return nil, errors.New("unexpected Asset Archive property value")
+				return nil, fmt.Errorf("unexpected Asset Archive property value for %q", key)
 			}
 			archive, isarchive, err := resource.DeserializeArchive(objmap)
 			if err != nil {
@@ -352,7 +380,7 @@ func UnmarshalPropertyValue(v *structpb.Value, opts MarshalOptions) (*resource.P
 			contract.Assert(isarchive)
 			if opts.ComputeAssetHashes {
 				if err = archive.EnsureHash(); err != nil {
-					return nil, errors.Wrapf(err, "failed to compute archive hash")
+					return nil, errors.Wrapf(err, "failed to compute archive hash for %q", key)
 				}
 			}
 			m := resource.NewArchiveProperty(archive)
@@ -360,21 +388,16 @@ func UnmarshalPropertyValue(v *structpb.Value, opts MarshalOptions) (*resource.P
 		case resource.SecretSig:
 			value, ok := obj["value"]
 			if !ok {
-				return nil, errors.New("malformed RPC secret: missing value")
+				return nil, fmt.Errorf("malformed RPC secret: missing value for %q", key)
 			}
-			if !opts.KeepSecrets {
-				logging.V(5).Infof("unmarshalling secret as raw value, as opts.KeepSecrets is false")
-				return &value, nil
-			}
-			s := resource.MakeSecret(value)
-			return &s, nil
+			return unmarshalSecretPropertyValue(value, opts), nil
 		case resource.ResourceReferenceSig:
 			urn, ok := obj["urn"]
 			if !ok {
-				return nil, errors.New("malformed resource reference: missing urn")
+				return nil, fmt.Errorf("malformed resource reference for %q: missing urn", key)
 			}
 			if !urn.IsString() {
-				return nil, errors.New("malformed resource reference: urn not a string")
+				return nil, fmt.Errorf("malformed resource reference for %q: urn not a string", key)
 			}
 
 			id, hasID := "", false
@@ -386,14 +409,14 @@ func UnmarshalPropertyValue(v *structpb.Value, opts MarshalOptions) (*resource.P
 				case idProp.IsComputed():
 					// Leave the ID empty to indicate that it is unknown.
 				default:
-					return nil, errors.New("malformed resource reference: id not a string")
+					return nil, fmt.Errorf("malformed resource reference for %q: id not a string", key)
 				}
 			}
 
 			var packageVersion string
 			if packageVersionProp, ok := obj["packageVersion"]; ok {
 				if !packageVersionProp.IsString() {
-					return nil, errors.New("malformed resource reference: packageVersion not a string")
+					return nil, fmt.Errorf("malformed resource reference for %q: packageVersion not a string", key)
 				}
 				packageVersion = packageVersionProp.StringValue()
 			}
@@ -406,7 +429,7 @@ func UnmarshalPropertyValue(v *structpb.Value, opts MarshalOptions) (*resource.P
 						v := structpb.Value{
 							Kind: &structpb.Value_StringValue{StringValue: UnknownStringValue},
 						}
-						return UnmarshalPropertyValue(&v, opts)
+						return UnmarshalPropertyValue(key, &v, opts)
 					}
 					value = id
 				}
@@ -421,12 +444,61 @@ func UnmarshalPropertyValue(v *structpb.Value, opts MarshalOptions) (*resource.P
 				ref = resource.MakeComponentResourceReference(resource.URN(urn.StringValue()), packageVersion)
 			}
 			return &ref, nil
+		case resource.OutputValueSig:
+			value, known := obj["value"]
+
+			var secret bool
+			if secretProp, ok := obj["secret"]; ok {
+				if !secretProp.IsBool() {
+					return nil, fmt.Errorf("malformed output value for %q: secret not a bool", key)
+				}
+				secret = secretProp.BoolValue()
+			}
+
+			if !opts.KeepOutputValues {
+				result := &value
+				if !known {
+					result, err = UnmarshalPropertyValue(key, &structpb.Value{
+						Kind: &structpb.Value_StringValue{StringValue: UnknownStringValue},
+					}, opts)
+					if err != nil {
+						return nil, err
+					}
+				}
+				if secret && result != nil {
+					result = unmarshalSecretPropertyValue(*result, opts)
+				}
+				return result, nil
+			}
+
+			var dependencies []resource.URN
+			if dependenciesProp, ok := obj["dependencies"]; ok {
+				if !dependenciesProp.IsArray() {
+					return nil, fmt.Errorf("malformed output value for %q: dependencies not an array", key)
+				}
+				dependencies = make([]resource.URN, len(dependenciesProp.ArrayValue()))
+				for i, dep := range dependenciesProp.ArrayValue() {
+					if !dep.IsString() {
+						return nil, fmt.Errorf(
+							"malformed output value for %q: element in dependencies not a string", key)
+					}
+					dependencies[i] = resource.URN(dep.StringValue())
+				}
+			}
+
+			output := resource.NewOutputProperty(resource.Output{
+				Element:      value,
+				Known:        known,
+				Secret:       secret,
+				Dependencies: dependencies,
+			})
+			return &output, nil
 		default:
-			return nil, errors.Errorf("unrecognized signature '%v' in property map", sig)
+			return nil, fmt.Errorf("unrecognized signature '%v' in property map for %q", sig, key)
 		}
 
 	default:
-		contract.Failf("Unrecognized structpb value kind in RPC[%s]: %v", opts.Label, reflect.TypeOf(v.Kind))
+		contract.Failf("Unrecognized structpb value kind in RPC[%s] for %q: %v", opts.Label, key, reflect.TypeOf(v.Kind))
 		return nil, nil
 	}
 }
@@ -457,6 +529,15 @@ func unmarshalUnknownPropertyValue(s string, opts MarshalOptions) (resource.Prop
 	return resource.PropertyValue{}, false
 }
 
+func unmarshalSecretPropertyValue(v resource.PropertyValue, opts MarshalOptions) *resource.PropertyValue {
+	if !opts.KeepSecrets {
+		logging.V(5).Infof("unmarshalling secret as raw value, as opts.KeepSecrets is false")
+		return &v
+	}
+	s := resource.MakeSecret(v)
+	return &s
+}
+
 // MarshalNull marshals a nil to its protobuf form.
 func MarshalNull(opts MarshalOptions) *structpb.Value {
 	return &structpb.Value{
@@ -502,7 +583,8 @@ func MarshalAsset(v *resource.Asset, opts MarshalOptions) (*structpb.Value, erro
 	// To marshal an asset, we need to first serialize it, and then marshal that.
 	sera := v.Serialize()
 	serap := resource.NewPropertyMapFromMap(sera)
-	return MarshalPropertyValue(resource.NewObjectProperty(serap), opts)
+	pk := resource.PropertyKey(v.URI)
+	return MarshalPropertyValue(pk, resource.NewObjectProperty(serap), opts)
 }
 
 // MarshalArchive marshals an archive into its wire form for resource provider plugins.
@@ -523,5 +605,6 @@ func MarshalArchive(v *resource.Archive, opts MarshalOptions) (*structpb.Value,
 	// To marshal an archive, we need to first serialize it, and then marshal that.
 	sera := v.Serialize()
 	serap := resource.NewPropertyMapFromMap(sera)
-	return MarshalPropertyValue(resource.NewObjectProperty(serap), opts)
+	pk := resource.PropertyKey(v.URI)
+	return MarshalPropertyValue(pk, resource.NewObjectProperty(serap), opts)
 }
